/*
Real-time Online/Offline Charging System (OCS) for Telecom & ISP environments
Copyright (C) ITsysCOM GmbH

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package services

import (
	"reflect"
	"sync"
	"testing"

	"github.com/cgrates/birpc"
	"github.com/cgrates/cgrates/config"
	"github.com/cgrates/cgrates/cores"
	"github.com/cgrates/cgrates/engine"
	"github.com/cgrates/cgrates/utils"
)

// TestSupplierSCoverage for cover testing
func TestSupplierSCoverage(t *testing.T) {
	cfg := config.NewDefaultCGRConfig()
	cfg.StatSCfg().Enabled = true
	filterSChan := make(chan *engine.FilterS, 1)
	filterSChan <- nil
	shdChan := utils.NewSyncedChan()
	chS := engine.NewCacheS(cfg, nil, nil)
	server := cores.NewServer(nil)
	srvDep := map[string]*sync.WaitGroup{utils.DataDB: new(sync.WaitGroup)}
	db := NewDataDBService(cfg, nil, false, srvDep)
	anz := NewAnalyzerService(cfg, server, filterSChan, shdChan, make(chan birpc.ClientConnector, 1), srvDep)
	supS := NewRouteService(cfg, db, chS, filterSChan, server, make(chan birpc.ClientConnector, 1), nil, anz, srvDep)

	if supS.IsRunning() {
		t.Errorf("Expected service to be down")
	}
	supS2 := &RouteService{
		cfg:         cfg,
		dm:          db,
		cacheS:      chS,
		filterSChan: filterSChan,
		server:      server,
		connMgr:     nil,
		routeS:      &engine.RouteService{},
		connChan:    make(chan birpc.ClientConnector, 1),
		anz:         anz,
		srvDep:      srvDep,
	}
	if !supS2.IsRunning() {
		t.Errorf("Expected service to be running")
	}
	serviceName := supS2.ServiceName()
	if !reflect.DeepEqual(serviceName, utils.RouteS) {
		t.Errorf("\nExpecting <%+v>,\n Received <%+v>", utils.RouteS, serviceName)
	}
	shouldRun := supS2.ShouldRun()
	if !reflect.DeepEqual(shouldRun, false) {
		t.Errorf("\nExpecting <false>,\n Received <%+v>", shouldRun)
	}
	cacheSrv, err := engine.NewService(chS)
	if err != nil {
		t.Fatal(err)
	}
	supS2.connChan <- cacheSrv
	supS2.Shutdown()
	if supS.IsRunning() {
		t.Errorf("Expected service to be down")
	}
}

func TestRoutesReload(t *testing.T) {
	routeS := &RouteService{}
	err := routeS.Reload()
	if err != nil {
		t.Errorf("Expected no error, got %v", err)
	}
}
